﻿Imports System.Text
Imports System.Threading

Public Class Form1

    ' Port Number
    Private m_strPortNum As String() = {"COM1", "COM2", "COM3", "COM4", "COM7"}
    ' BaudRate
    Private m_strBaudRate As String() = {"110", "300", "600", "1200", "2400", "4800", "9600", "14400", "19200", "38400", "56000", "57600", "115200", "128000"}
    ' Parity Bit
    Private m_strParity As String() = {"None", "Odd", "Even", "Mark", "Space"}
    ' Stop Bit
    Private m_strStopBits As String() = {"None", "One", "Tow", "OnePointFive"}

    ' Thread 선언
    Private m_readThread As Thread
    Private Shared m_bStop As Boolean

    '  AutoResetEvent 객체. 대기중인 스레드에게 외부에서 신호를 보내 스레드의 흐름을 통제
    '  인자로 false를 사용하기 때문에 비신호상태로 선언. 즉, 이벤트는 대기상태로 선언
    Private Shared m_StopCompletionEvent As AutoResetEvent = New AutoResetEvent(False)

    ' 스레드에서 UI 컨트롤을 접근하기 위한 delegate 함수 선언및 정의
    Delegate Sub ListRecCallback(ByVal strtext As String)

    ' 실제 UI에 접근하는 함수. 델리게이트의 인자로 사용
    Private Sub RecListAdd(ByVal strRecData As String)
        lstRecvData.AddItem(strRecData)
    End Sub

    ' 수신데이터를 ListBox에 추가하는 함수
    Private Sub DelegateRecListAdd(ByVal strRecData As String)
        ' Invoke 메서드는 델리게이트 Type을 파라미터로 받아들임. 컴파일 에러방지
        Me.Invoke(New ListRecCallback(AddressOf RecListAdd), strRecData)
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        ' 현재 Form을 MainForm으로 설정합니다.
        SmartForm1.MainForm = Me

        ' 통신 포트
        cbPortNum.DataSource = m_strPortNum
        cbPortNum.SelectItemIndex = 1

        ' 통신 속도
        cbBaudRate.DataSource = m_strBaudRate
        cbBaudRate.SelectItemIndex = 6

        ' 패리티 값
        cbParity.DataSource = m_strParity
        cbParity.SelectItemIndex = 0

        ' 스톱비트
        cbStopBit.DataSource = m_strStopBits
        cbStopBit.SelectItemIndex = 1

        labSendData.Text = "This is a Test Message"
    End Sub

    ' 포트 오픈 및 수신 스레드 시작
    Private Sub btnPortOpen_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnPortOpen.Click
        If SerialPort1.IsOpen = False Then
            ' 통신포트 사용자 선택
            SerialPort1.PortName = cbPortNum.Text

            ' 통신속도 사용자 선택
            SerialPort1.BaudRate = Convert.ToInt32(cbBaudRate.Text)

            ' 패리티 사용자 선택
            SerialPort1.Parity = CType(cbParity.SelectItemIndex, System.IO.Ports.Parity)

            ' 데이터 비트 사용자 선택
            SerialPort1.DataBits = Convert.ToInt32(labDataBits.Text)

            ' 스톱비트 사용자 선택
            SerialPort1.StopBits = CType(cbStopBit.SelectItemIndex, System.IO.Ports.StopBits)

            ' 읽기 작업을 마쳐야 하는 제한 시간(ms)을 가져오거나 설정
            SerialPort1.ReadTimeout = 500

            SerialPort1.Open()

            ' 수신 스레드 시작
            m_readThread = New Thread(New ThreadStart(AddressOf ComPortReadFunc))
            m_readThread.Start()
        End If
    End Sub

    ' Client 연결 및 수신 스레드 함수
    Private Sub ComPortReadFunc()
        Dim iRecCnt As Integer = 0
        Dim strRecData As String

        ' 바이트 배열 수신 버퍼 선언
        Dim recBuff As Byte() = New Byte(SerialPort1.ReadBufferSize - 1) {}

        m_bStop = False
        Dim i As Integer = 0

        Try
            While True
                Try
                    For i = 0 To SerialPort1.ReadBufferSize - 1
                        ' ReadByte()를 통해서 읽은 데이터를 수신 버퍼에 저장
                        recBuff(i) = Convert.ToByte(SerialPort1.ReadByte())
                    Next

                Catch tExp As TimeoutException
                    iRecCnt = i
                End Try

                If iRecCnt <> 0 Then
                    ' 수신된 바이트 데이터를 유니코드 문자열로 변환
                    strRecData = Encoding.ASCII.GetString(recBuff, 0, iRecCnt)

                    ' 수신된 데이터를 ListBox에 추가한다.
                    DelegateRecListAdd(strRecData)
                    ' Application.DoEvents()
                End If

                ' 스레드를 종료하기위한 검사. 시리얼 포트를 닫는 경우 TRUE로 설정됨
                If m_bStop = True Then
                    Exit While
                End If

                Thread.Sleep(100)
            End While

            ' 스레드에 대기 해제 신호 발생. 스레드 완벽하게 종료됨
            m_StopCompletionEvent.Set()
        Catch exp As Exception

            If m_bStop = True Then
                ' serial Port를 접근시 Try~Catch 로 항상 처리한다. 
                ' Open이 안되거나 생성이 안된경우 무조건 예외를 발생할수있다.
                exp.ToString()

                ' 스레드에 대기 해제 신호 발생. 스레드 완벽하게 종료됨
                m_StopCompletionEvent.Set()
                Return
            End If
        End Try
    End Sub

    Private Sub SerialPortClose()
        If SerialPort1.IsOpen = False Then
            Return
        End If

        m_bStop = True

        SerialPort1.ReadTimeout = 5000

        ' 수신 스레드가 종료되기를 기다린다.
        Thread.Sleep(1000)

        ' 스레드 STOP을 처리하는 함수에서 스레드 함수가 완전히 종료될때까지
        ' 스레드 STOP처리를 함수의 Return 즉, Blocking 하도록 처리
        If m_StopCompletionEvent.WaitOne(CInt(1000), False) = True Then

            Try
                ' 시리얼 포트 닫기
                SerialPort1.Close()
            Catch exp As Exception
                exp.ToString()
            End Try

            Return
        End If

        ' SmartTimer 시작
        AutoSendTimer.Start()
        ' 시리얼 포트 종료
        SerialPort1.Close()
    End Sub

    ' 포트 닫기
    Private Sub btnPortClose_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnPortClose.Click
        SerialPortClose()
    End Sub

    ' 데이터 자동으로 송신하기
    Private Sub AutoSendTimer_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles AutoSendTimer.Tick
        If SerialPort1.IsOpen = False Then
            AutoSendTimer.Stop()
            Return
        End If

        Try
            Dim sendBuff As Byte() = New Byte(1023) {}

            ' 입력된 Text를 ASCII 코드로 전환
            sendBuff = Encoding.ASCII.GetBytes(labSendData.Text)

            ' 데이터 전송
            SerialPort1.Write(sendBuff, 0, labSendData.Text.Length)

            If chkAutoSend.Checked = False Then
                AutoSendTimer.Stop()
            End If

        Catch exp As Exception
            ' serialPort1의 접근시 Try~Catch 로 항상 처리한다. 
            ' Open이 안되거나 생성이 안된경우 무조건 예외를 발생할수있다.
            exp.ToString()
            AutoSendTimer.Stop()
            SerialPort1.Close()
            m_bStop = True
        End Try
    End Sub

    ' 데이터 전송 시작
    Private Sub btnSendStart_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSendStart.Click
        AutoSendTimer.Interval = 1000
        AutoSendTimer.Start()
    End Sub

    ' 데이터 전송 정지
    Private Sub btnSendStop_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSendStop.Click
        AutoSendTimer.Stop()
    End Sub

    ' 송신 메시지를 변경하기 위한 SmartKeyboard 출력
    Private Sub labSendData_Click(ByVal sender As Object, ByVal e As EventArgs) Handles labSendData.Click
        smartKeyboard1.Show()
        smartKeyboard1.TargetInputObject = labSendData
    End Sub
End Class
