﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace TechNote46_SerialPort_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Port Number
        private string[] m_strPortNum = { "COM1", "COM2", "COM3", "COM4", "COM7" };
        // BaudRate
        private string[] m_strBaudRate = { "110", "300", "600", "1200", "2400", "4800", "9600", "14400", "19200", "38400", "56000", "57600", "115200", "128000" };
        // Parity Bit
        private string[] m_strParity = { "None", "Odd", "Even", "Mark", "Space" };
        // Stop Bit
        private string[] m_strStopBits = { "None", "One", "Tow", "OnePointFive" };

        // Thread 선언
        private Thread m_readThread;
        private static bool m_bStop;

        //  AutoResetEvent 객체. 대기중인 스레드에게 외부에서 신호를 보내 스레드의 흐름을 통제
        //  인자로 false를 사용하기 때문에 비신호상태로 선언. 즉, 이벤트는 대기상태로 선언
        private static AutoResetEvent m_StopCompletionEvent = new AutoResetEvent(false);

        // 스레드에서 UI 컨트롤을 접근하기 위한 delegate 함수 선언및 정의
        delegate void ListRecCallback(string strtext);

        // 실제 UI에 접근하는 함수. 델리게이트의 인자로 사용
        private void RecListAdd(string strRecData)
        {
            lstRecvData.AddItem(strRecData);
        }

        // 수신데이터를 ListBox에 추가하는 함수
        private void DelegateRecListAdd(string strRecData)
        {
            // Invoke 메서드는 델리게이트 Type을 파라미터로 받아들임. 컴파일 에러방지
            this.Invoke(new ListRecCallback(RecListAdd), strRecData);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            // 통신 포트
            cbPortNum.DataSource = m_strPortNum;
            cbPortNum.SelectItemIndex = 1;

            // 통신 속도
            cbBaudRate.DataSource = m_strBaudRate;
            cbBaudRate.SelectItemIndex = 6;

            // 패리티 값
            cbParity.DataSource = m_strParity;
            cbParity.SelectItemIndex = 0;

            // 스톱비트
            cbStopBit.DataSource = m_strStopBits;
            cbStopBit.SelectItemIndex = 1;

            labSendData.Text = "This is a Test Message";
        }

        // 포트 오픈 및 수신 스레드 시작
        private void btnPortOpen_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen == false)
            {
                // 통신포트 사용자 선택
                serialPort1.PortName = cbPortNum.Text;

                // 통신속도 사용자 선택
                serialPort1.BaudRate = Convert.ToInt32(cbBaudRate.Text);

                // 패리티 사용자 선택
                serialPort1.Parity = (System.IO.Ports.Parity)cbParity.SelectItemIndex;

                // 데이터 비트 사용자 선택
                serialPort1.DataBits = Convert.ToInt32(labDataBits.Text);

                // 스톱비트 사용자 선택
                serialPort1.StopBits = (System.IO.Ports.StopBits)cbStopBit.SelectItemIndex;

                // 읽기 작업을 마쳐야 하는 제한 시간(ms)을 가져오거나 설정
                serialPort1.ReadTimeout = 500;

                serialPort1.Open();

                // 수신 스레드 시작
                m_readThread = new Thread(new ThreadStart(ComPortReadFunc));
                m_readThread.Start();
            }
        }

        // Client 연결 및 수신 스레드 함수
        private void ComPortReadFunc()
        {
            int iRecCnt = 0;
            string strRecData;

            // 바이트 배열 수신 버퍼 선언
            byte[] recBuff = new byte[serialPort1.ReadBufferSize];

            m_bStop = false;
            int i = 0;

            try
            {
                while (true)
                {
                    try
                    {
                        for (i = 0; i < serialPort1.ReadBufferSize; i++)
                        {
                            // ReadByte()를 통해서 읽은 데이터를 수신 버퍼에 저장
                            recBuff[i] = Convert.ToByte(serialPort1.ReadByte());
                        }
                    }
                    catch (TimeoutException tExp)
                    {
                        iRecCnt = i;
                    }

                    if (iRecCnt != 0)
                    {
                        // 수신된 바이트 데이터를 유니코드 문자열로 변환
                        strRecData = Encoding.ASCII.GetString(recBuff, 0, iRecCnt);

                        // 수신된 데이터를 ListBox에 추가한다.
                        DelegateRecListAdd(strRecData);
                        // Application.DoEvents();
                    }


                    // 스레드를 종료하기위한 검사. 시리얼 포트를 닫는 경우 TRUE로 설정됨
                    if (m_bStop == true)
                    {
                        break;
                    }
                    Thread.Sleep(100);
                }

                // 스레드에 대기 해제 신호 발생. 스레드 완벽하게 종료됨
                m_StopCompletionEvent.Set();

            }
            catch (Exception exp)
            {
                if (m_bStop == true)
                {
                    // serial Port를 접근시 Try~Catch 로 항상 처리한다. 
                    // Open이 안되거나 생성이 안된경우 무조건 예외를 발생할수있다.
                    exp.ToString();

                    // 스레드에 대기 해제 신호 발생. 스레드 완벽하게 종료됨
                    m_StopCompletionEvent.Set();
                    return;
                }
            }
        }

        private void SerialPortClose()
        {
            if (serialPort1.IsOpen == false)
            {
                return;
            }

            m_bStop = true;

            serialPort1.ReadTimeout = 5000;

            // 수신 스레드가 종료되기를 기다린다.
            Thread.Sleep(1000);

            // 스레드 STOP을 처리하는 함수에서 스레드 함수가 완전히 종료될때까지
            // 스레드 STOP처리를 함수의 Return 즉, Blocking 하도록 처리
            if (m_StopCompletionEvent.WaitOne((int)1000, false) == true)
            {
                try
                {
                    // 시리얼 포트 닫기
                    serialPort1.Close();
                }
                catch (Exception exp)
                {
                    exp.ToString();
                }
                return;
            }

            // SmartTimer 시작
            AutoSendTimer.Start();
            // 시리얼 포트 종료
            serialPort1.Close();
        }

        // 포트 닫기
        private void btnPortClose_Click(object sender, EventArgs e)
        {
            SerialPortClose();
        }

        // 데이터 자동으로 송신하기
        private void AutoSendTimer_Tick(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen == false)
            {
                AutoSendTimer.Stop();
                return;
            }

            try
            {
                byte[] sendBuff = new byte[1024];

                // 입력된 Text를 ASCII 코드로 전환
                sendBuff = Encoding.ASCII.GetBytes(labSendData.Text);

                // 데이터 전송
                serialPort1.Write(sendBuff, 0, labSendData.Text.Length);

                if (chkAutoSend.Checked == false)
                {
                    AutoSendTimer.Stop();
                }
            }
            catch (Exception exp)
            {
                // serialPort1의 접근시 Try~Catch 로 항상 처리한다. 
                // Open이 안되거나 생성이 안된경우 무조건 예외를 발생할수있다.
                exp.ToString();
                AutoSendTimer.Stop();
                serialPort1.Close();
                m_bStop = true;
            }
        }

        // 데이터 전송 시작
        private void btnSendStart_Click(object sender, EventArgs e)
        {
            AutoSendTimer.Interval = 1000;
            AutoSendTimer.Start();
        }

        // 데이터 전송 정지
        private void btnSendStop_Click(object sender, EventArgs e)
        {
            AutoSendTimer.Stop();
        }

        // 송신 메시지를 변경하기 위한 SmartKeyboard 출력
        private void labSendData_Click(object sender, EventArgs e)
        {
            smartKeyboard1.Show();
            smartKeyboard1.TargetInputObject = labSendData;
        }
    }
}